@@ -264,6 +264,7 @@ shishi_cfg (Shishi * handle, char *option)
 		struct servent *se;
 		char *p;
 		int protocol = UDP;
+		int port = -1;
 
 		if ((p = strchr (value, '/')))
 		  {
@@ -280,6 +281,13 @@ shishi_cfg (Shishi * handle, char *option)
 				   "Ignoring unknown KDC parameter: %s", p);
 		  }
 
+		if ((p = strchr (value, ':')))
+		  {
+		    *p = '\0';
+		    p++;
+		    port = atoi (p);
+		  }
+
 		he = gethostbyname (value);	/* XXX move to netio.c */
 		if (he == NULL ||
 		    he->h_addr_list[0] == NULL || he->h_addrtype != AF_INET)
@@ -302,11 +310,16 @@ shishi_cfg (Shishi * handle, char *option)
 
 		sinaddr->sin_family = he->h_addrtype;
 		memcpy (&sinaddr->sin_addr, he->h_addr_list[0], he->h_length);
-		se = getservbyname ("kerberos", NULL);
-		if (se)
-		  sinaddr->sin_port = se->s_port;
+		if (port == -1)
+		  {
+		    se = getservbyname ("kerberos", NULL);
+		    if (se)
+		      sinaddr->sin_port = se->s_port;
+		    else
+		      sinaddr->sin_port = htons (88);
+		  }
 		else
-		  sinaddr->sin_port = htons (88);
+		  sinaddr->sin_port = htons(port);
 	      }
 	  if (realm)
 	    break;
@@ -501,6 +514,23 @@ shishi_cfg_clientkdcetype (Shishi * handle, int32_t ** etypes)
   return handle->nclientkdcetypes;
 }
 
+/**
+ * shishi_cfg_clientkdcetype_fast:
+ * @handle: Shishi library handle create by shishi_init().
+ *
+ * Extract the default etype from the list of preferred client etypes.
+ *
+ * Return value: Return the default encryption types.
+ **/
+int32_t
+shishi_cfg_clientkdcetype_fast (Shishi * handle)
+{
+  if (handle->nclientkdcetypes > 0)
+    return handle->clientkdcetypes[0];
+  else
+    return SHISHI_AES256_CTS_HMAC_SHA1_96;
+}
+
 /**
  * shishi_cfg_clientkdcetype_set:
  * @handle: Shishi library handle create by shishi_init().
