@@ -727,7 +727,8 @@ shishi_kdcrep_decrypt (Shishi * handle,
   free (cipher);
   if (res != SHISHI_OK)
     {
-      shishi_error_printf (handle, "KDCRep decryption failed, wrong password?");
+      shishi_error_printf (handle,
+			   "KDCRep decryption failed, wrong password?");
       return res;
     }
 
