@@ -27,7 +27,7 @@
 #include <time.h>		/* time_t */
 #include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.8"
+#define SHISHI_VERSION "0.0.9"
 
 /* Error codes */
 typedef enum
@@ -598,6 +598,7 @@ extern const char *shishi_cfg_default_systemfile (Shishi * handle);
 extern const char *shishi_cfg_default_userdirectory (Shishi * handle);
 extern const char *shishi_cfg_default_userfile (Shishi * handle);
 extern int shishi_cfg_clientkdcetype (Shishi * handle, int32_t ** etypes);
+extern int32_t shishi_cfg_clientkdcetype_fast (Shishi * handle);
 extern int shishi_cfg_clientkdcetype_set (Shishi * handle, char *value);
 extern int shishi_cfg_authorizationtype_set (Shishi * handle, char *value);
 
@@ -625,6 +626,10 @@ extern char *shishi_principal_default_guess (void);
 extern const char *shishi_principal_default (Shishi * handle);
 extern void shishi_principal_default_set (Shishi * handle,
 					  const char *principal);
+extern int shishi_principal_name (Shishi * handle,
+				  Shishi_asn1 namenode,
+				  const char *namefield,
+				  char **out, size_t * outlen);
 extern int shishi_principal_name_get (Shishi * handle, Shishi_asn1 namenode,
 				      const char *namefield, char *out,
 				      size_t * outlen);
@@ -648,6 +653,8 @@ extern char *shishi_server_for_local_service (Shishi * handle,
 					      const char *service);
 
 /* ticket.c */
+extern int shishi_ticket_server (Shishi * handle, Shishi_asn1 ticket,
+				 char **server, size_t * serverlen);
 extern int shishi_ticket_sname_get (Shishi * handle,
 				    Shishi_asn1 ticket,
 				    char *server, size_t * serverlen);
@@ -731,6 +738,7 @@ extern int shishi_tkt_renew_till (Shishi_tkt * tkt,
 				  char *renewtilltime, size_t * renewtilllen);
 extern time_t shishi_tkt_renew_tillc (Shishi_tkt * tkt);
 extern int shishi_tkt_keytype (Shishi_tkt * tkt, int32_t * etype);
+extern int32_t shishi_tkt_keytype_fast (Shishi_tkt * tkt);
 extern int shishi_tkt_keytype_p (Shishi_tkt * tkt, int32_t etype);
 extern int shishi_tkt_server (Shishi_tkt * tkt,
 			      char *service, size_t * servicelen);
@@ -740,6 +748,7 @@ extern int shishi_tkt_server_realm (Shishi_tkt * tkt,
 				    size_t * serverrealmlen);
 extern int shishi_tkt_valid_at_time_p (Shishi_tkt * tkt, time_t now);
 extern int shishi_tkt_valid_now_p (Shishi_tkt * tkt);
+extern int shishi_tkt_expired_p (Shishi_tkt * tkt);
 extern int shishi_tkt_decrypt (Shishi_tkt * tkt, Shishi_key * key);
 extern void shishi_tkt_done (Shishi_tkt * tkt);
 extern int shishi_tkt_flags (Shishi_tkt * tkt, int *flags);
@@ -909,6 +918,10 @@ extern int
 shishi_authenticator_add_random_subkey (Shishi * handle,
 					Shishi_asn1 authenticator);
 extern int
+shishi_authenticator_add_random_subkey_etype (Shishi * handle,
+					      Shishi_asn1 authenticator,
+					      int etype);
+extern int
 shishi_authenticator_add_subkey (Shishi * handle,
 				 Shishi_asn1 authenticator,
 				 Shishi_key * subkey);
@@ -1023,12 +1036,16 @@ extern int shishi_asreq_cnamerealm_get (Shishi * handle, Shishi_asn1 kdcreq,
 					size_t * cnamerealmlen);
 extern int shishi_kdcreq_nonce (Shishi * handle, Shishi_asn1 kdcreq,
 				uint32_t * nonce);
+extern int shishi_kdcreq_client (Shishi * handle, Shishi_asn1 kdcreq,
+				 char **client, size_t *clientlen);
 extern int shishi_kdcreq_cname_get (Shishi * handle,
 				    Shishi_asn1 kdcreq,
 				    char *cname, size_t * cnamelen);
 extern int shishi_kdcreq_set_cname (Shishi * handle, Shishi_asn1 kdcreq,
 				    Shishi_name_type name_type,
 				    const char *principal);
+extern int shishi_kdcreq_server (Shishi * handle, Shishi_asn1 kdcreq,
+				 char **server, size_t ** serverlen);
 extern int shishi_kdcreq_sname_get (Shishi * handle, Shishi_asn1 kdcreq,
 				    char *sname, size_t * snamelen);
 extern int shishi_kdcreq_snamerealm_get (Shishi * handle, Shishi_asn1 kdcreq,
@@ -1037,6 +1054,8 @@ extern int shishi_kdcreq_snamerealm_get (Shishi * handle, Shishi_asn1 kdcreq,
 extern int shishi_kdcreq_set_sname (Shishi * handle, Shishi_asn1 kdcreq,
 				    Shishi_name_type name_type,
 				    const char *sname[]);
+extern int shishi_kdcreq_realm (Shishi * handle, Shishi_asn1 kdcreq,
+				char **realm, size_t * realmlen);
 extern int shishi_kdcreq_realm_get (Shishi * handle, Shishi_asn1 kdcreq,
 				    char *realm, size_t * realmlen);
 extern int shishi_kdcreq_set_realm (Shishi * handle, Shishi_asn1 kdcreq,
@@ -1314,7 +1333,6 @@ extern int shishi_aes_cts (Shishi * handle, int decryptp,
 extern int shishi_cipher_supported_p (int type);
 extern const char *shishi_cipher_name (int type);
 extern int shishi_cipher_blocksize (int type);
-extern int shishi_cipher_minpadsize (int type);
 extern int shishi_cipher_confoundersize (int type);
 extern size_t shishi_cipher_keylen (int type);
 extern size_t shishi_cipher_randomlen (int type);
@@ -1485,6 +1503,9 @@ extern int shishi_asn1_read_bitstring (Shishi * handle, Shishi_asn1 node,
 extern int shishi_asn1_read_optional (Shishi * handle,
 				      Shishi_asn1 node, const char *field,
 				      char *data, size_t * datalen);
+extern int shishi_asn1_read2_optional (Shishi * handle,
+				       Shishi_asn1 node, const char *field,
+				       char **data, size_t * datalen);
 extern int shishi_asn1_empty_p (Shishi * handle, Shishi_asn1 node,
 				const char *field);
 extern int shishi_asn1_write (Shishi * handle, Shishi_asn1 node,
@@ -1572,8 +1593,9 @@ extern Shishi_asn1 shishi_der2asn1_encprivpart (Shishi * handle,
 
 /* ap.c */
 extern int shishi_ap (Shishi * handle, Shishi_ap ** ap);
-extern void shishi_ap_done (Shishi_ap * ap);
+extern int shishi_ap_etype (Shishi * handle, Shishi_ap ** ap, int etype);
 extern int shishi_ap_nosubkey (Shishi * handle, Shishi_ap ** ap);
+extern void shishi_ap_done (Shishi_ap * ap);
 extern int shishi_ap_set_tktoptions (Shishi_ap * ap,
 				     Shishi_tkt * tkt, int options);
 extern int shishi_ap_tktoptions (Shishi * handle,
@@ -1679,13 +1701,13 @@ extern int shishi_key_random (Shishi * handle,
 			      int32_t type, Shishi_key ** key);
 extern int shishi_key_from_value (Shishi * handle,
 				  int32_t type,
-				  char *value, Shishi_key ** key);
+				  const char *value, Shishi_key ** key);
 extern int shishi_key_from_base64 (Shishi * handle,
 				   int32_t type,
-				   char *value, Shishi_key ** key);
+				   const char *value, Shishi_key ** key);
 extern int shishi_key_from_random (Shishi * handle,
 				   int32_t type,
-				   char *random, size_t randomlen,
+				   const char *random, size_t randomlen,
 				   Shishi_key ** outkey);
 extern int shishi_key_from_string (Shishi * handle,
 				   int32_t type,
@@ -1871,9 +1893,15 @@ extern int shishi_encticketpart_get_enc_part_etype (Shishi * handle,
 extern int shishi_encticketpart_get_key (Shishi * handle,
 					 Shishi_asn1 encticketpart,
 					 Shishi_key ** key);
+extern int shishi_encticketpart_crealm (Shishi * handle,
+					Shishi_asn1 encticketpart,
+					char **crealm, size_t * crealmlen);
 extern int shishi_encticketpart_crealm_set (Shishi * handle,
 					    Shishi_asn1 encticketpart,
 					    const char *realm);
+extern int shishi_encticketpart_client (Shishi * handle,
+					Shishi_asn1 encticketpart,
+					char **client, size_t * clientlen);
 extern int shishi_encticketpart_cname_get (Shishi * handle,
 					   Shishi_asn1 encticketpart,
 					   char *cname, size_t * cnamelen);
